<?php
/**
 * @file
 * Contains \Drupal\group_member_import\Form\GroupMemberImportSettingsForm.
 */

namespace Drupal\group_member_import\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\group\Entity\Group;
use Drupal\group\Entity\GroupInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Session\AccountInterface;
use Drupal\user\Entity\User;
use Drupal\group_member_import\GroupMemberImportFields;
use Drupal\Core\Form\ConfigFormBase;
use Symfony\Component\Routing\RouteCollection;
use Symfony\Component\Routing\Route;
use Drupal\block\Entity\Block;
use Drupal\block\BlockInterface;

/**
 * Provides the settings form.
 */
class GroupMemberImportSettingsForm extends ConfigFormBase {

  /** 
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'group_member_import.settings';  

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'group_member_import_settings_form';
  }

  /** 
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }  


  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $config = $this->config(static::SETTINGS);

    $helper_service = \Drupal::service('group_member_import.helper');
    $fields = $helper_service->getAllAvailableFieldsPerEntity();  

    $default_values_fields = $config->get('active_fields');
    $default_values_allowed_roles =  $config->get('allowed_roles');
    $default_values_display_block = $config->get('display_block');

    $form['label'] = [
      '#type' => 'html_tag',
      '#tag' => 'p',
      '#value' => $this->t('Check all fields you do not want to import.'),
    ];

    if (is_array($fields) && !empty($fields)) {
     
      foreach ($fields as $key => $field) {
        if (is_array($field) && !empty($field)) {           
          $options = array_combine($field, $field);          
          $form[$key] = array(
            '#type' => 'details',
            '#title' => $key,
            '#open' => FALSE, // Controls the HTML5 'open' attribute. Defaults to FALSE.
          );
          $form[$key]['check'] = [
            '#type' => 'checkboxes',
            '#title' => $key,
            '#options' => $options,
            '#default_value' => $default_values_fields
          ];
        }
      }
    }

    $allowed_roles = \Drupal::entityQuery('user_role')->accessCheck(FALSE)->execute();
    foreach($allowed_roles as $key => $role) {
      if ($key === 'administrator' || $key === 'authenticated' || $key === 'anonymous') {
        unset($allowed_roles[$key]);
      }
    }

    $form['allowed_roles'] = array(
      '#type' => 'checkboxes',
      '#title' => t('Allowed Roles'),
      '#options' => $allowed_roles,
      '#default_value' => $default_values_allowed_roles
    );


    if (empty($default_values_display_block)) {
      // In case of open social we want membership our location.
      $moduleHandler = \Drupal::service('module_handler');
      if ($moduleHandler->moduleExists('social')) {
        $default_values_display_block = 'membership';
      }      
      else {
        $default_values_display_block = 'members';  
      }
    }    

    // On wich route you want to publish the 
    // group member import button block
    $form['display_block'] = [
      '#type' => 'textfield',
      '#title' => t('Display group member import button on specific group path'),
      '#default_value' => $default_values_display_block,
      '#required' => TRUE,
      '#description' => $this->t('Enter a valid group path without any slash.')
    ];

    $form['submit'] = [
      '#type'  => 'submit',
      '#value' => $this->t('Save'),
    ];

    $form['#tree'] = TRUE;

 

    return $form;

  }

    /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    $display_block = $form_state->getValue('display_block');

    $route_pattern = 'group/{group}/' . $display_block;

    //dump($route_pattern);

    /* @var \Drupal\Core\Routing\RouteProviderInterface $route_provider */
    $route_provider = \Drupal::service('router.route_provider');
    $route_collection = $route_provider->getRoutesByPattern($route_pattern);

    if ($route_collection->count() === 0) {
      $form_state->setErrorByName('display_block', $this->t('This route within a group context does not exists.'));
    }


  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    // Retrieve the configuration.
    $config = $this->config(static::SETTINGS);    

    $values = $form_state->cleanValues()->getValues();
    $allowed_roles = $form_state->cleanValues()->getValue('allowed_roles'); 

    // Delete all current roles, before adding new ones
    $config->set('allowed_roles', []);

    foreach ($allowed_roles as $key => $value) {
      if ($key === $value) {
        $save_roles[$key] = $key;
        $config->set('allowed_roles', $save_roles);
        $config->save();
      }
    }

    // What we do not need we place here.
    $not_needed_keys = [
      'allowed_roles',
      'display_block'
    ];

    // Set active fields for import.
    foreach ($values as $key => $value) {
      if (is_array($value) && !in_array($key, $not_needed_keys)) {
        foreach ($value['check'] as $index => $checked) {
          if ($index === $checked) {
            $save_fields[] = $index;
            $config->set('active_fields', $save_fields);
            $config->save();
          }
        }
      }
    }


    // Block from settings.
    $display_block = $values['display_block'];

    // Define where we want to display our button block.
    $block = Block::load('groupmemberimport_block');
    if ($block instanceof BlockInterface) {

      $visibility = $block->getVisibility();


      if (!empty($display_block)) {

        // Save into config
        $config->set('display_block', $display_block);
        $config->save();

        // Alter block
        $block = Block::load('groupmemberimport_block');
        $visibility['request_path']['pages'] = "/group/*/$display_block";
        $block->setVisibilityConfig('request_path', $visibility['request_path']);
        $block->save();

      }

    }

  }

  /**
   * Checks access for a specific request.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Run access checks for this account.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(AccountInterface $account, GroupInterface $group = NULL) {
    // Check permissions and combine that with any custom access checking needed. Pass forward
    // parameters from the route and/or request as needed.

    $user = User::load($account->id());

    //kint($group);

    if ($group) {

      $member = $group->getMember($account);

      if ($member) {
        if($member->hasPermission('edit group', $account)) {
          return AccessResult::allowed();
        }
      }
      elseif ($user->hasRole('administrator')) {
        return AccessResult::allowed();
      }

    }
    else {
      if ($user->hasRole('administrator')) {
        return AccessResult::allowed();
      }
    }

    return AccessResult::forbidden();

  }


}
