<?php declare(strict_types = 1);

namespace Drupal\group_member_import\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\profile\Entity\ProfileInterface;
use Drupal\node\Entity\Node;
use Drupal\file\Entity\File;
use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\taxonomy\Entity\Term;
use Drupal\user\Entity\User;
use Drupal\user\UserInterface;

/**
 * @todo Add class description.
 */
final class Helper {

  /**
   * Constructs a Helper object.
   */
  public function __construct(
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly EntityFieldManagerInterface $entityFieldManager,
    private readonly EntityTypeBundleInfoInterface $entityTypeBundleInfo,
    private readonly AccountProxyInterface $currentUser,
    private readonly ConfigFactoryInterface $configFactory,
  ) {}

  /**
   * Checks if data already exists
   *
   * @param Drupal\Core\Session\AccountInterface $account
   *   The account.
   * @param string $profile_type_id
   *   The profile type id to check for.
   * @return bool TRUE | FALSE
   *   Returns true or Drupal\profile\Entity\ProfileInterface $profile.
   */
  public function checkIfAlreadyExistsProfile(AccountInterface $account, string $profile_type_id) {

    $already_exists = FALSE;

    $profile = $this->entityTypeManager->getStorage('profile')->loadByUser($account, $profile_type_id);

    if ($profile instanceof ProfileInterface) {
      $already_exists = $profile;
    }

    return $already_exists;

  }

  /**
   * getFieldType.
   *
   * @param string $bundle
   *   The bundle for the entity
   * @param string $field
   *   The field name to get information for
   * @return array $options_array
   *   Returns a array of field definitions
   */
  public function getFieldType($bundle, $field) {

    if ($bundle === 'user') {
      $entity_type = 'user';
    }
    else {
      $entity_type = 'profile';
    }

    $options_array = [];
    $definitions = $this->entityFieldManager->getFieldDefinitions($entity_type, $bundle);

    if (isset($definitions[$field])) {
      $options_array = $definitions[$field]->getType();
    }

    return $options_array;

  }

  public function getTermReferenceFromHierachy($voc, $term_data) {

    $term_id = NULL;
    $found_terms = [];

    $vocName = strtolower($voc);
    $vid = preg_replace('@[^a-z0-9_]+@', '_', $vocName);
    $vocabularies = Vocabulary::loadMultiple();
    /* Create Vocabulary if it is not exists */
    if (!isset($vocabularies[$vid])) {
      return $termIds;
    }

    $term_names = $term_data;
    
    foreach ($term_names as $key => $term_name) {
      if ($key === 0) {
        $terms = $this->getTaxonomyTermEntitiesByName($term_name, $vid);
        foreach ($terms as $tid => $term) {
          $childs_first = $this->getTermChildren($tid, $vid);
          foreach($childs_first as $child_first_id => $child_first_value) {
            if (in_array($child_first_value, $term_names)) {
              $childs_second = $this->getTermChildren($child_first_id, $vid);
              foreach ($childs_second as $child_second_id => $child_second_value) {
                if (in_array($child_second_value, $term_names)) {
                  $term_id = $child_second_id;
                }
              }
            }  
          }
          
          
        }
      }
    }

    return $term_id;

  }



  function getTermChildren($parent_id, $vocabulary): array {

    $children = [];

    $child_terms = $this->entityTypeManager->getStorage('taxonomy_term')->loadTree(
      $vocabulary,  
      $parent_id,  
      1,  
      FALSE
    );
  
    $children = []; 
  
    if(!empty($child_terms)) {  
      foreach ($child_terms as $term) {    
        $children[$term->tid] = $term->name;  
      } 
    }
  
    return $children;
  
  }   


  protected function getTaxonomyTermEntitiesByName($term_name, $vid) {
    $conditions = ['name' => $term_name, 'vid' => $vid];
    $terms = $this->entityTypeManager->getStorage('taxonomy_term')->loadByProperties($conditions);
    return $terms;
  }

  /**
   * getTermReference.
   *
   * @param string $voc
   *   The vocabulary for the taxonomy
   * @param string $term_name
   *   The term name
   * @param string $field
   *   The field to search a term by (if null, search by term name)
   * @param string $single
   *   If field references only a single term (will not be parsed via ",")
   * @return array $terms_array
   *   Returns a array of term ids
   */
  public function getTermReference($voc, $term_name, $field = null, $single = true) {

    $vocName = strtolower($voc);
    $vid = preg_replace('@[^a-z0-9_]+@', '_', $vocName);
    $vocabularies = Vocabulary::loadMultiple();
    /* Create Vocabulary if it is not exists */
    if (!isset($vocabularies[$vid])) {
      $this->create_voc($vid, $voc);
    }

    if ($single) {
      $termArray = array(trim($term_name));
    } 
    else {
      $termArray = array_map('trim', explode(',', $term_name));
    }

    $termIds = [];
    foreach ($termArray as $term) {
      $term_id = $this->get_term_id($term, $vid, $field);
      if (empty($term_id) && !$field) {
        $term_id = $this->create_term($voc, $term, $vid);
      }
      $termIds[]['target_id'] = $term_id;
    }

    return $termIds;

  }

  /**
   * Create Vocabulary.
   *
   * @param $vid
   *   The vid of the vocabulary
   * @param $voc
   *   The vocabulry name
   * @return void
   */
  protected function create_voc($vid, $voc): void {
    $vocabulary = Vocabulary::create(['vid' => $vid, 'machine_name' => $vid, 'name' => $voc, ]);
    $vocabulary->save();
  }

  /**
   * Create Term.
   *
   * @param $voc
   *   Vocabulary the term belongs to.
   * @param $term
   *   Term name.
   * @param $vid
   *   The ID of the vocabulry.
   * @return void
   */
  protected function create_term($voc, $term, $vid) {
    Term::create(['parent' => [$voc], 'name' => $term, 'vid' => $vid, ])->save();
    $termId = $this->get_term_id($term, $vid);
    return $termId;
  }

  /**
   * Get term id.
   *
   * @param $term 
   *   The term name.
   * @param $vid
   *   The ID of the vocabulry.
   * @param [type] $field
   *   The field.
   * @return 
   *   FALSE or ID of Term.
   */
  protected function get_term_id($term, $vid, $field = null) {

    $is_term = FALSE;
    $properties = [];    
    $properties['vid'] = $vid;    

    if ($field) {
      $properties[$field] = $term;    
    } 
    else {
      $properties['name'] = $term;    
    }

    $terms = $this->entityTypeManager->getStorage('taxonomy_term')->loadByProperties($properties);
    $term = reset($terms);

    if ($term) {
      $is_term = $term->id();
    } 

    return $is_term;

  }

  protected function get_parent_term($term_id) {
    $parents = $this->entityTypeManager->getStorage('taxonomy_term')->loadParents($term_id);
    return $parents;
  }

  protected function get_taxonomy_term_by_id($term_id) {
    $term = $this->entityTypeManager->getStorage('taxonomy_term')->load($term_id);
    return $term;  
  }

  /**
   * Get node id.
   *
   * @param string $title
   *   The node title.
   * @return 
   *   The found node reference(s).
   */
  public function get_node_id(string $title) {
    $nodeReference = [];
    $db = \Drupal::database();
    foreach ($title as $key => $value) {
      $query = $db->select('node_field_data', 'n');
      $query->fields('n', ['nid']);
      $nodeId = $query->condition('n.title', trim($value))->execute()->fetchField();
      $nodeReference[$key]['target_id'] = $nodeId;
    }

    return $nodeReference;

  }

  /**
   * Get users.
   *
   * @param string $name
   *   The user name.
   * @return 
   *   The found user reference(s).
   */
  public function get_users_id(string $name) {
    $userReference = [];
    $db = \Drupal::database();
    foreach ($name as $key => $value) {
      $query = $db->select('users_field_data', 'n');
      $query->fields('u', ['uid']);
      $userId = $query->condition('u.name', trim($value))->execute()->fetchField();
      $userReference[$key]['target_id'] = $userId;
    }

    return $userReference;

  }  

  /**
   * Get group id.
   *
   * @param string $title
   *   The group title.
   * @return 
   *   The found group reference(s).
   */
  public function get_group_id(string $title) {
    $gid = NULL;
    $db = \Drupal::database();
    $query = $db->select('groups_field_data', 'g');
    $query->fields('g', ['id']);
    $groupId = $query->condition('g.label', trim($title))->execute()->fetchField();
    if ($groupId) {
      $gid = $groupId;  
    }
    return $gid;
  }  

  /**
   * Get user id.
   *
   * @param string $name
   *   The user name.
   * @return
   *   The user id.
   */
  public function get_user_id(string $name) {
    $user_id = \Drupal::database()->select('users_field_data', 'u')->fields('u', ['uid'])->condition('u.name', trim($name))->execute()->fetchField();
    return $user_id;
  }

  public function get_user_roles(array $user_roles) {
    
  } 

  /**
   * Get field settings.
   *
   * @param string $bundle
   *   The bundle of an entity.
   * @param string $field
   *   The field of an entity.
   * @return 
   *   Array of field definitions.
   */
  public function getFieldSettings(string $bundle, string $field) {

    if ($bundle === 'user') {
      $entity_type = 'user';
    }
    else {
      $entity_type = 'profile';
    }

    $options_array = [];
    $definitions = $this->entityFieldManager->getFieldDefinitions($entity_type, $bundle);

    if (isset($definitions[$field])) {
      $options_array = $definitions[$field]->getSettings();
    }

    return $options_array;

  }  

  /**
   * Get user name from email.
   *
   * @param string $email
   *   Email of the user.
   * @return 
   *   Username of the user.
   */
  public function getUserNameFromEmail(string $email) {
    $array = explode("@", $email);
    $username = $array[0];

    if (user_load_by_name($username)) {
      $random_number = random_int(2, 6);
      $username = $username . '_' . $random_number;
    }

    return $username;

  }


  /**
   * Generate a unique Drupal username.
   *
   * @param string $email
   *   The email.
   *
   * @return string
   *   A unique username.
   */
  function generateUniqueUsernameByEmail(string $email): string {

    $array = explode("@", $email);
    $name = $array[0];

    // Check if the exact base name is available.
    if (!user_load_by_name($name)) {
      return $name;
    }

    // Append incremental suffix until a free one is found.
    $suffix = 1;
    while (user_load_by_name($name . '_' . $suffix)) {
      $suffix++;
      if ($suffix > 1000) {
        throw new \Exception('Unable to generate a unique username.');
      }
    }

    return $name . '_' . $suffix;

  }


  /**
   * Get available account fields.
   *
   * @return array
   *   Field definitions.
   */
  public function getAvailableAccountFields() {

    $fields = [];

    $field_definitions = $this->entityFieldManager->getFieldDefinitions('user', 'user');

    if ($field_definitions) {
      foreach ($field_definitions as $field_definition) {
        if (!empty($field_definition->getTargetBundle())) {
          $fields['name'][] = $field_definition->getName();
          $fields['type'][] = $field_definition->getType();
          $fields['setting'][] = $field_definition->getSettings();
        }
      }
    }

    return $fields;

  }

  /**
   * Get all available fields
   * of user and profile.
   *
   * @return array
   *   Field column definitions.
   */
  public function getAllAvailableFields() {

    $columns = [];
    $fields = [];
    $allowedFields = $this->getAllowedAccountBaseFields();

    // Get attached fields
    foreach (\Drupal::service('entity_field.manager')->getFieldDefinitions('user', 'user') as $field_definition) {
      $definition[] = $field_definition->getName();
      if (isset($definition) && !empty($definition)) {
        foreach ($definition as $field) {
          $columns = 'user.' . $field;
        }
      }
      $fields[] = $columns;
    }

    // Get Profile fields
    $profile_types = \Drupal::service('entity_type.bundle.info')->getBundleInfo('profile');
    foreach ($profile_types as $bundle => $profile) {
      foreach (\Drupal::service('entity_field.manager')->getFieldDefinitions('profile', $bundle) as $field_definition) {
        $fields[] = $this->getColumnDefinitions($field_definition->getName() , $bundle);
      }
    }

    return $fields;

  }  

  /**
   * Get all available fields
   * per entity.
   *
   * @return array
   *   Field column definitions.
   */
  public function getAllAvailableFieldsPerEntity() {

    $columns = [];
    $fields = [];
    $allowedFields = $this->getAllowedAccountBaseFields();

    // Get attached fields
    foreach (\Drupal::service('entity_field.manager')->getFieldDefinitions('user', 'user') as $field_definition) {

      $definition[] = $field_definition->getName();

      if (isset($definition) && !empty($definition)) {
        foreach ($definition as $field) {

          $columns = 'user.' . $field;

        }
      }

      $fields['user'][] = $columns;

    }

    // Get Profile fields
    $profile_types = \Drupal::service('entity_type.bundle.info')->getBundleInfo('profile');

    foreach ($profile_types as $bundle => $profile) {

      foreach (\Drupal::service('entity_field.manager')->getFieldDefinitions('profile', $bundle) as $field_definition) {

        $profile_columns = $this->getColumnDefinitions($field_definition->getName() , $bundle);

        if (!is_array($profile_columns)) {
          $fields[$bundle][] = $profile_columns;
        }
      }
    }

    return $fields;

  }  

  /**
   * Get column definitions.
   *
   * @param $field_definition_names
   *   The field definition name.
   * @param $bundle
   *   The bundle of the entity.
   * @return array
   *   The columns of the Entity.
   */
  protected function getColumnDefinitions($field_definition_names, $bundle) {

    $columns = [];
    $definition[] = $field_definition_names;
    $allowedFields = $this->getAllowedAccountBaseFields();

    if (isset($definition) && !empty($definition)) {
      foreach ($definition as $field) {

        if (strpos($field, 'field') !== false) {

          $columns = $bundle . '.' . $field;

        }

      }
    }

    return $columns;

  }
  
  /**
   * Check if valid email
   *
   * @param $email
   *   The email address.
   * @return boolean
   *   Valid email TRUE | FALSE
   */
  public function checkIfValidEmail($email) {
    $is_valid = FALSE;
    if (\Drupal::service('email.validator')->isValid($email)) {
      $is_valid = TRUE;
    }   
    return $is_valid;
  }

  /**
   * Get allowed account base fields.
   *
   * @return array
   *   Allowed account base fields.
   */
  protected function getAllowedAccountBaseFields() {
    return ['name', 'pass', 'mail', 'timezone', 'status', 'langcode'];
  }  

}
