<?php declare(strict_types = 1);

namespace Drupal\group_member_import\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\group_member_import\Service\Helper;
use Drupal\profile\Entity\ProfileInterface;
use Drupal\node\Entity\Node;
use Drupal\file\Entity\File;
use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\taxonomy\Entity\Term;
use Drupal\user\Entity\User;

/**
 * @todo Add class description.
 */
final class ImportFields {

  /**
   * Constructs an ImportFields object.
   */
  public function __construct(
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly AccountProxyInterface $currentUser,
    private readonly ConfigFactoryInterface $configFactory,
    private readonly Helper $helper,
  ) {}

  /**
   * Import user
   *
   * @param $userArray
   *   Array of user fields.
   * @param $profileArray
   *   Array of profile fields.
   * @param $group
   *   The group id.
   * @param $group_roles
   *   The needed group roles.
   * @param $user_roles
   *   The needed user roles.
   * @return void
   */
  public function importUser($userArray, $profileArray, $group, $group_roles, $user_roles): void {

    $active_group_roles = [];

    if (is_array($group_roles) && !empty($group_roles)) {
      $active_group_roles = ['group_roles' => array_keys($group_roles) ];
    }

    $usermail = FALSE;

    // Get email from mail, uid or username, and delete these fields from user array.
    if (array_key_exists('mail', $userArray)) {
      $usermail = $userArray['mail'];
      unset($userArray['mail']);
    }
    if (array_key_exists('uid', $userArray)) {         
      if ($user = User::load($userArray['uid'])) {
        $usermail = $user->mail->value;
      }     
      unset($userArray['uid']);
    }
    if (array_key_exists('name', $userArray)) {
      $username = $userArray['name'];
      if ($user = user_load_by_name($userArray['name'])) {
        $usermail = $user->mail->value;
      }
      unset($userArray['name']);
    }
    if (array_key_exists('init', $userArray)) {
      $usermail = $userArray['init'];      
      unset($userArray['init']);
    }     
    
    // We need the usermail to proceed
    if (is_string($usermail) && $this->helper->checkIfValidEmail($usermail)) {

      // Check if we need to update an existing record
      $account = user_load_by_mail($usermail);
      // Account = FALSE
      if (!$account) {
        // Insert new user
        $user = User::create($userArray);
        $user->uid = '';

        if (isset($username) && is_string($username)) {
          $user->setUsername($username);
        }
        else {
          $username = $this->helper->generateUniqueUsernameByEmail($usermail);
          $user->setUsername($username);          
        }

        $user->setEmail($usermail);
        $user->set("init", $usermail);  

        if (is_array($user_roles) && !empty($user_roles)) {        

          foreach ($user_roles as $key => $role) {
            if ($key != 'authenticated' || $key != 'anonymous') {
              $user->addRole($key);
            }
          }
        }

        $user->enforceIsNew();
        $user->activate();
        $user->save();
        $uid = $user->id();
        
        $current_user = User::load($uid);

        foreach ($profileArray as $profile_id => $profile_fields) {                    

          $profile = $this->helper->checkIfAlreadyExistsProfile($current_user, $profile_id);

          if (!$profile instanceof ProfileInterface) {
            $data = [
              'uid' => $current_user->id(),
              'type' => $profile_id,
            ];
            $profile = $this->entityTypeManager->getStorage('profile')->create($data);
          }

          if (is_array($profile_fields) && !empty($profile_fields)) {
            foreach ($profile_fields as $field_name => $field_value) {
              $profile->set($field_name, $field_value);
            }
            $profile->save();            
          }

        }        

        // Attach the imported member to his group
        if ($group) {
          $group_storage = $this->entityTypeManager->getStorage('group')->load($group);
          $group_storage->addMember($user, $active_group_roles);
        }

      }
      else {
        // Update User
        $user = $account;

        foreach ($userArray as $field_name => $field_value) {
          if ($field_name == 'pass') {
            $config = $this->configFactory->get('group_member_import.settings');          
            // Change password for existing users only if specified in form.
            if ($config->get('overwrite_passwords') == TRUE) {
              $user->setPassword($field_value);
            }
          }
          else {
            $user->set($field_name, $field_value);
          }
        }

        if (is_array($user_roles) && !empty($user_roles)) {
          foreach ($user_roles as $key => $role) {
            if ($key != 'authenticated' || $key != 'anonymous') {
              $user->addRole($key);
            }
          }
        }  

        foreach ($profileArray as $profile_id => $profile_fields) {                    

          $profile = $this->helper->checkIfAlreadyExistsProfile($user, $profile_id);

          if (!$profile instanceof ProfileInterface) {
            $data = [
              'uid' => $user->id(),
              'type' => $profile_id,
            ];
            $profile = $this->entityTypeManager->getStorage('profile')->create($data);
          }

          if (is_array($profile_fields) && !empty($profile_fields)) {
            foreach ($profile_fields as $field_name => $field_value) {
              $profile->set($field_name, $field_value);
            }
            $profile->save();            
          }

        }                

        if ($group) {
          $group_storage = $this->entityTypeManager->getStorage('group')->load($group);
          $group_storage->addMember($user, $active_group_roles);
        }

        $user->save();

      }

    }

  }

}
